/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/*******************************************************************************
 *
 *  drv3901-q1-RegMap.h - Declaration file for utility functions and global variables
 *  DRV3901-Q1EVM_FW
 *  09/02/2022
 *
 ******************************************************************************/
#ifndef DRV3901_REGMAP_H_
#define DRV3901_REGMAP_H_

#include "../dataTypeDefinition.h"

/*******************************************************************************
 * MACROs
 *******************************************************************************/

/*******************************************************************************
 * DRV3901-Q1 SPI Register ADDRESSES
 *******************************************************************************/
#define SPI_REG_STATUS0        (uint16_t)(0x01)       // STATUS0 Register
#define SPI_REG_STATUS1        (uint16_t)(0x02)       // STATUS1 Register
#define SPI_REG_STATUS2        (uint16_t)(0x03)       // STATUS2 Register
#define SPI_REG_STATUS3        (uint16_t)(0x04)       // STATUS3 Register       //PG2p0 change
#define SPI_REG_MEAS0          (uint16_t)(0x05)       // MEAS1 Register
#define SPI_REG_MEAS1          (uint16_t)(0x06)       // MEAS2 Register
#define SPI_REG_MEAS2          (uint16_t)(0x07)       // MEAS3 Register         //PG2p0 change
#define SPI_REG_CONFIG0        (uint16_t)(0x10)       // CONFIG1 Register
#define SPI_REG_CONFIG1        (uint16_t)(0x11)       // CONFIG2 Register
#define SPI_REG_CONFIG2        (uint16_t)(0x12)       // CONFIG3 Register
#define SPI_REG_CONFIG3        (uint16_t)(0x13)       // CONFIG4 Register
#define SPI_REG_CONFIG4        (uint16_t)(0x14)       // CONFIG5 Register
#define SPI_REG_CMD0           (uint16_t)(0x1C)       // CMD_CONFIG Register
#define SPI_REG_CMD1           (uint16_t)(0x1D)       // CRC_PROTECTED_CMD Register
#define SPI_REG_CMD2           (uint16_t)(0x1E)       // CMD_CRC Register

/*******************************************************************************
 * Structure
 *******************************************************************************/

//  Structure variables for the DRV3901-Q1 registers.
typedef struct DRV3901_REG
{
    // starting address = 0x09C0, Number of bytes = 26 bytes
    uint16_t status0_reg;
    uint16_t status1_reg;
    uint16_t status2_reg;
    uint16_t status3_reg;
    uint16_t meas0_reg;
    uint16_t meas1_reg;
    uint16_t meas2_reg;
    uint16_t config0_reg;
    uint16_t config1_reg;
    uint16_t config2_reg;
    uint16_t config3_reg;
    uint16_t config4_reg;
    uint16_t cmd0_reg;
    uint16_t cmd1_reg;
    uint16_t cmd2_reg;
} DRV3901_Q1_REG_t;

#endif // DRV3901_REGMAP_H_
