/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

/********************************************************************************
 *
 *  drv3901.c - Definition file for utility functions
 *  DRV3901-Q1EVM_FIRMWARE
 *  09/02/2022
 *  Murugavel Raju
 *
 *******************************************************************************/
#define FIRMWARE_VERSION 1.00                              // Updated to RTM release version


#include "drv3901-q1/drv3901-q1.h"
#include "crc.h"
#include "spi.h"
#include "uart.h"
#include "Serial_Cmd_Monitor.h"

/********************************************************************************
 * Data Structures exported for GUI Composer
 *******************************************************************************/
#pragma DATA_SECTION(gBrushedObj,".drvVarSection");
#pragma DATA_SECTION(gDeviceSpiReg,".drvRegSection");
volatile DRV3901_Q1_REG_t gDeviceSpiReg;
volatile DRV3901_Q1_Device_Obj_t gBrushedObj;

/*******************************************************************************
 *  Variable definitions
 *******************************************************************************/
volatile uint8_t configx[10] = {0,0,0,0,0,0,0,0,0,0};      // Initialize array for CONFIGx upper and lower bytes

/*******************************************************************************
 * API IMPLEMENTATION
 *******************************************************************************/
// gui_deviceControlUpdateTask - Synchronize values in the global data structure written to by
// the GUI with the hardware.
void gui_deviceControlUpdateTask()
{
    // Fault LEDs update
    if (read_FAULT_A_pin)
    {
        P2OUT &= ~FAULT_A_LED;                             // FAULT_A LED off
        gBrushedObj.DRVA_FAULT = 1;                        // GUI fault indicator
    }
    else
    {
        P2OUT |= FAULT_A_LED;                              // FAULT_A LED on
        gBrushedObj.DRVA_FAULT = 0;                        // GUI fault indicator
    }

    if (read_FAULT_B_pin)
    {
        P2OUT &= ~FAULT_B_LED;                             // FAULT_B LED off
        gBrushedObj.DRVB_FAULT = 1;                        // GUI fault indicator
    }
    else
    {
        P2OUT |= FAULT_B_LED;                              // FAULT_B LED on
        gBrushedObj.DRVB_FAULT = 0;                        // GUI fault indicator
    }
    // End fault LEDs update

    // Trigger1 function
    if (gBrushedObj.TRG1 && TRG1state)                                  // Trigger1 enabled
    {
        TRG1state = 0;
        switch (gBrushedObj.TRG1a)
        {
        case 0:                                            // Trigger1 high
            P4SEL &= ~TRIG1;                               // change pin to gpio mode
            TB0CTL = TBCLR;                                // Timer stop and reset
            P4OUT |= TRIG1;                                // output high
            P4DIR |= TRIG1;
            break;
        case 1:                                            // Trigger1 16kHz PWM 75% duty cycle
            TRIG1_pwm16k(75);
            break;
        case 2:                                            // Trigger1 16kHz PWM 25% duty cycle
            TRIG1_pwm16k(25);
            break;
        case 3:                                            // Trigger1 125kHz PWM 75% duty cycle
            TRIG1_pwm125k(75);
            break;
        case 4:                                            // Trigger1 125kHz PWM 25% duty cycle
            TRIG1_pwm125k(25);
            break;
        case 5:                                           // Trigger1 low
            P4SEL &= ~TRIG1;                               // change pin to gpio mode
            TB0CTL = TBCLR;                                // Timer stop and reset
            P4OUT &= ~TRIG1;                               // output low
            P4DIR |= TRIG1;
            break;
        }
    }
    else if (!gBrushedObj.TRG1 && !TRG1state)                            // Trigger1 disabled to low
    {
        TRG1state = 1;
        P4SEL &= ~TRIG1;                                   // change pin to gpio mode
        TB0CTL = TBCLR;                                    // Timer stop and reset
        P4OUT &= ~TRIG1;                                   // output low
        P4DIR |= TRIG1;
    }
    // End Trigger1 function

    // Trigger2 function
    if (gBrushedObj.TRG2 && TRG2state)                                  // Trigger2 enabled
    {
        TRG2state = 0;
        switch (gBrushedObj.TRG2a)
        {
        case 0:                                            // Trigger2 high
            P1SEL &= ~TRIG2;                               // change pin to gpio mode
            TA0CTL = TACLR;                                // Timer stop and reset
            P1OUT |= TRIG2;
            P1DIR |= TRIG2;
            break;
        case 1:                                            // Trigger2 16kHz PWM 75% duty cycle
            TRIG2_pwm16k(75);
            break;
        case 2:                                            // Trigger2 16kHz PWM 25% duty cycle
            TRIG2_pwm16k(25);
            break;
        case 3:                                            // Trigger2 125kHz PWM 75% duty cycle
            TRIG2_pwm125k(75);
            break;
        case 4:                                            // Trigger2 125kHz PWM 25% duty cycle
            TRIG2_pwm125k(25);
            break;
        case 5:                                           // Trigger2 low
            P1SEL &= ~TRIG2;                               // change pin to gpio mode
            TA0CTL = TACLR;                                // Timer stop and reset
            P1OUT &= ~TRIG2;
            P1DIR |= TRIG2;
            break;
        }
    }
    else if (!gBrushedObj.TRG2 && !TRG2state)                            // Trigger2 disabled to low
    {
        TRG2state = 1;
        P1SEL &= ~TRIG2;                                   // change pin to gpio mode
        TA0CTL = TACLR;                                    // Timer stop and reset
        P1OUT &= ~TRIG2;
        P1DIR |= TRIG2;
    }
    // End Trigger2 function

} // end gui_deviceControlUpdateTask()

void gui_regRead(void)
{
    gDeviceSpiReg.status0_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS0);
    gDeviceSpiReg.status1_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS1);
    gDeviceSpiReg.status2_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS2);
    gDeviceSpiReg.status3_reg = (uint16_t) spi_readRegister(SPI_REG_STATUS3);   //PG2p0 change
    gDeviceSpiReg.meas0_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS0);
    gDeviceSpiReg.meas1_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS1);
    gDeviceSpiReg.meas2_reg = (uint16_t) spi_readRegister(SPI_REG_MEAS2);       //PG2p0 change
    gDeviceSpiReg.config0_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIG0);
    gDeviceSpiReg.config1_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIG1);
    gDeviceSpiReg.config2_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIG2);
    gDeviceSpiReg.config3_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIG3);
    gDeviceSpiReg.config4_reg = (uint16_t) spi_readRegister(SPI_REG_CONFIG4);
    gDeviceSpiReg.cmd0_reg = (uint16_t) spi_readRegister(SPI_REG_CMD0);
    gDeviceSpiReg.cmd1_reg = (uint16_t) spi_readRegister(SPI_REG_CMD1);
    gDeviceSpiReg.cmd2_reg = (uint16_t) spi_readRegister(SPI_REG_CMD2);
} // end gui_regRead()

// This function handles synchronizing the device registers with the GUI
void gui_regUpdateTask(void)
{
    volatile uint8_t drvRegAddr;
    volatile uint16_t drvDataNew;

    // Write to single register using manual register write command
    if (gBrushedObj.ManWriteCmd)
    {
        drvRegAddr = gBrushedObj.ManWriteAddr;
        drvDataNew = gBrushedObj.ManWriteData;

        // This function will update the register global variable for the specific register that is manually written
        switch (drvRegAddr)
        {
        case SPI_REG_CONFIG0:
            gDeviceSpiReg.config0_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG1:
            gDeviceSpiReg.config1_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG2:
            gDeviceSpiReg.config2_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG3:
            gDeviceSpiReg.config3_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG4:
            gDeviceSpiReg.config4_reg = drvDataNew;
            break;
        case SPI_REG_CMD0:
            gDeviceSpiReg.cmd0_reg = drvDataNew;
            break;
        case SPI_REG_CMD1:
            gDeviceSpiReg.cmd1_reg = drvDataNew;
            break;
        case SPI_REG_CMD2:
            gDeviceSpiReg.cmd2_reg = drvDataNew;
            break;
        default:
            break;
        }

        // update upper and lower byte values of ConfigAx and ConfigBx registers from
        // their respective global variables for crc calculation
        configx[0] = (uint8_t) (gDeviceSpiReg.config0_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIG0 upper byte
        configx[1] = (uint8_t) (gDeviceSpiReg.config0_reg) & 0xFF;                       // CONFIG0 lower byte
        configx[2] = (uint8_t) (gDeviceSpiReg.config1_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIG1 upper byte
        configx[3] = (uint8_t) (gDeviceSpiReg.config1_reg) & 0xFF;                       // CONFIG1 lower byte
        configx[4] = (uint8_t) (gDeviceSpiReg.config2_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIG2 upper byte
        configx[5] = (uint8_t) (gDeviceSpiReg.config2_reg) & 0xFF;                       // CONFIG2 lower byte
        configx[6] = (uint8_t) (gDeviceSpiReg.config3_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIG3 upper byte
        configx[7] = (uint8_t) (gDeviceSpiReg.config3_reg) & 0xFF;                       // CONFIG3 lower byte
        configx[8] = (uint8_t) (gDeviceSpiReg.config4_reg >> SPI_UPPER_DATA_POS) & 0xFF; // CONFIG4 upper byte
        configx[9] = (uint8_t) (gDeviceSpiReg.config4_reg) & 0xFF;                       // CONFIG4 lower byte

        spi_writeRegister((uint8_t) drvRegAddr, (uint16_t) drvDataNew);
        gBrushedObj.ManWriteCmd = false;
    }

    // Read all registers at once
    if (ReadCMD)
    {
        gui_regRead();
        ReadCMD = 0;
    }

    // Read single register using manual register read command
    if (gBrushedObj.ManReadCmd)
    {
        drvRegAddr = gBrushedObj.ManReadAddr;
        drvDataNew = spi_readRegister((uint8_t) drvRegAddr);

        gBrushedObj.ManReadData = drvDataNew;
        switch (drvRegAddr)
        {
        case SPI_REG_STATUS0:
            gDeviceSpiReg.status0_reg = drvDataNew;
            break;
        case SPI_REG_STATUS1:
            gDeviceSpiReg.status1_reg = drvDataNew;
            break;
        case SPI_REG_STATUS2:
            gDeviceSpiReg.status2_reg = drvDataNew;
            break;
        case SPI_REG_STATUS3:               //PG2p0 change
                   gDeviceSpiReg.status3_reg = drvDataNew;
                   break;
        case SPI_REG_MEAS0:
            gDeviceSpiReg.meas0_reg = drvDataNew;
            break;
        case SPI_REG_MEAS1:
            gDeviceSpiReg.meas1_reg = drvDataNew;
            break;
        case SPI_REG_MEAS2:                 //PG2p0 change
                   gDeviceSpiReg.meas1_reg = drvDataNew;
                   break;
        case SPI_REG_CONFIG0:
            gDeviceSpiReg.config0_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG1:
            gDeviceSpiReg.config1_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG2:
            gDeviceSpiReg.config2_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG3:
            gDeviceSpiReg.config3_reg = drvDataNew;
            break;
        case SPI_REG_CONFIG4:
            gDeviceSpiReg.config4_reg = drvDataNew;
            break;
        case SPI_REG_CMD0:
            gDeviceSpiReg.cmd0_reg = drvDataNew;
            break;
        case SPI_REG_CMD1:
            gDeviceSpiReg.cmd1_reg = drvDataNew;
            break;
        case SPI_REG_CMD2:
            gDeviceSpiReg.cmd2_reg = drvDataNew;
            break;
        default:
            break;
        }
        gBrushedObj.ManReadCmd = false;
    }
} // end gui_regUpdateTask()

/********************************************************************************
 * LOCAL ROUTINES
 *******************************************************************************/
void getEVMID(void)
{
    // Manual entry
    gBrushedObj.deviceID = 3901;                           // device is drv3901-q1
    gBrushedObj.revID = REV_1p0;                           // silicon revision 1p0
} // end getEVMID()

// MSP430G2955 Clock initialization
static inline void clockInit(void)
{
    // 16 MHz DCO for MCLK
    DCOCTL = 0x00;
    BCSCTL1= CALBC1_16MHZ;
    DCOCTL= CALDCO_16MHZ;

    // 16MHz SMCLK for PWM frequency, SPI clock and UART baud rate
    BCSCTL2 |= DIVS_0;
    BCSCTL3 |= LFXT1S_2;
} // end clockInit()

// MSP430G2955 GPIO initialization
static inline void gpioInit()
{
    // Port 1
    P1OUT = 0;
    P1DIR = 0;
    P1DIR |= (MCU_STATUS | TP13 | TP14 | TRIG2 | TP6);
    P1SEL = 0;
    P1SEL2 = 0;

    // Port 2
    P2OUT = 0;
    P2DIR = 0;
    P2DIR |= (FAULT_A_LED | FAULT_B_LED);
    P2SEL = 0;
    P2SEL2 = 0;

    // Port 3
    P3OUT = 0;
    P3DIR = (GUI_STATUS | TP33);
    P3SEL2 = 0;

    // Port 4
    P4OUT = 0;
    P4DIR = 0;
    P4DIR |= (DRV_A_LED | TRIG1 | DRV_B_LED | TP17 | TP20 | TP21 | TP31 | TP32);
    P4SEL = 0;
    P4SEL2 = 0;

    // Setup PUSH button for triggering IRQ
    // External pull up resistor 47kOhms R18
    P1DIR &= ~PUSH;                                        // input mode
    P1IES |= PUSH;                                         // falling edge
    P1IFG &= ~PUSH;                                        // clear IRQ that may have been generated by setting IES
    // P1IE  |= PUSH;                                         // enable IRQ on PUSH button press **not enabled in this firmware version**
} // end gpioInit()

// Setup Watchdog Timer as an Interval Timer, Timer Interval to periodically wake up MSP430G2955 from LPM0
static inline void watchdogTimerInit(void)
{
    WDTCTL = WDT_ADLY_1_9;                                 // Configure time interval for periodic wake up and processing 1.9 ms if using 32 kHz ACLK, with VLO ~4 ms
    IE1 |= WDTIE;                                          // Watchdog interval timer Interrupt enable
} // end watchdogTimerInit()

void mcu_init(void)
{
    // Stop Watchdog Timer
    WDTCTL = WDTPW | WDTHOLD;

    // Setup CLKs, GPIO and Peripherals
    gpioInit();                                            // MSP430G2955 GPIOI initialization
    clockInit();                                           // MSP430G2955 basic clock initialization
    uartInit();                                            // MSP430G2955 UART initialization
    spiInit();                                             // MSP430G2955 SPI initialization
    watchdogTimerInit();
} // end initMcu()

void drv3901q1evm_init(void)
{
    __delay_cycles(200000);                                // Power on settling delay 10ms

    // EVM Firmware version configuration
    gBrushedObj.firmwareVersion = FIRMWARE_VERSION;
    gBrushedObj.drv_SPI_mode = 1;                          // 0: peer to peer SPI, 1: Addressable SPI
    gBrushedObj.DRVA_SEL = 1;                              // 0: device not selected, 1: device selected
    gBrushedObj.DRVB_SEL = 0;                              // 0: device not selected, 1: device selected
    TRG1state = 0;
    TRG2state = 0;
    _delay_cycles(100);                                    // 6.25 us settling delay
    while (!(P3IN & SDO))
    {
        __delay_cycles(6000000);                           // Delay 400ms
        P2OUT ^= (FAULT_A_LED | FAULT_B_LED);              // Toggle FAULT LEDs to indicate VBAT = 0
    }
    __delay_cycles(6000000);                               // Delay 400ms
    P2OUT &= (FAULT_A_LED | FAULT_B_LED);                  // FAULT LEDs off after VBAT is on
    drv_RESET();                                           // Driver clear fault to clear POR flag
    RegReadPeriod = 0;                                     // Initialize Register read period
    ReadCMD = 0;                                           // Initialize Register read command
    gBrushedObj.AutoRead = 1;                              // Enable GUI Auto Read

    // Read device ID
    getEVMID();                                            // Get driver device ID
} // end drv3901q1evm_init()

inline void drv_RESET(void)
{
    spi_writeRegister(SPI_REG_CMD2, 0xC000);               // write CLR_FAULT and RE_INIT_NAD bits high
    __delay_cycles(20000);                                 // 1 ms settling delay
    spi_writeRegister(SPI_REG_CMD2, 0x8000);               // write CLR_FAULT high and clear RE_INIT_NAD bit
    __delay_cycles(100);                                   // 5 us settling delay
} // end drv_RESET()

// Trigger1 PWM 16kHz 25% or 75% duty cycle
void TRIG1_pwm16k(uint8_t duty)
{
    // TimerB0 for Trigger1
    TB0CTL = TBSSEL_2 | MC_0 | ID_0 | TBCLR;               // SMCLK = 16MHz, divider =1
    TB0CCR0 = 999;                                         // TimerB0 period value, /1000 for 16kHz

    if (duty == 25)                                        // 25% duty cycle
        TB0CCR1   = 249;                                   // TimerB0.1 compare value for duty cycle
    else if (duty == 75)                                   // 75% duty cycle
        TB0CCR1   = 749;                                   // TimerB0.1 compare value for duty cycle

    TB0CCTL1  |= OUTMOD_7;                                 // Output mode to PWM reset/set
    TB0CTL |= MC_1 | TACLR;                                // Timer up mode to start PWM
    P4SEL |= TRIG1;                                        // Output PWM on gpio
}// end Trigger1 PWM 16kHz 25% or 75% duty cycle

// Trigger1 PWM 125kHz 25% or 75% duty cycle
void TRIG1_pwm125k(uint8_t duty)
{
    // TimerB0 for Trigger1
    TB0CTL = TBSSEL_2 | MC_0 | ID_0 | TBCLR;               // SMCLK = 16MHz, divider =1
    TB0CCR0 = 127;                                         // TimerB0 period value, /128 for 125kHz

    if (duty == 25)                                        // 25% duty cycle
        TB0CCR1   = 31;                                    // TimerB0.1 compare value for duty cycle
    else if (duty == 75)                                   // 75% duty cycle
        TB0CCR1   = 95;                                    // TimerB0.1 compare value for duty cycle

    TB0CCTL1  |= OUTMOD_7;                                 // Output mode to PWM reset/set
    TB0CTL |= MC_1 | TACLR;                                // Timer up mode to start PWM
    P4SEL |= TRIG1;                                        // Output PWM on gpio
}// end Trigger1 PWM 125kHz 25% or 75% duty cycle

// Trigger2 PWM 16kHz 25% or 75% duty cycle
void TRIG2_pwm16k(uint8_t duty)
{
    // TimerA0 for Trigger2
    TA0CTL = TASSEL_2 | MC_0 | ID_0 | TACLR;               // SMCLK = 16MHz, divider =1
    TA0CCR0 = 999;                                         // TimerA0 period value, /1000 for 16kHz

    if (duty == 25)                                        // 25% duty cycle
        TA0CCR1   = 249;                                   // TimerA0.1 compare value for duty cycle
    else if (duty == 75)                                   // 75% duty cycle
        TA0CCR1   = 749;                                   // TimerA0.1 compare value for duty cycle

    TA0CCTL1  |= OUTMOD_7;                                 // Output mode to PWM reset/set
    TA0CTL |= MC_1 | TACLR;                                // Timer up mode to start PWM
    P1SEL |= TRIG2;                                        // Output PWM on gpio
}// end Trigger2 PWM 16kHz 25% or 75% duty cycle

// Trigger2 PWM 125kHz 25% or 75% duty cycle
void TRIG2_pwm125k(uint8_t duty)
{
    // TimerA0 for Trigger2
    TA0CTL = TASSEL_2 | MC_0 | ID_0 | TACLR;               // SMCLK = 16MHz, divider =1
    TA0CCR0 = 127;                                         // TimerA0 period value, /128 for 125kHz

    if (duty == 25)                                        // 25% duty cycle
        TA0CCR1   = 31;                                    // TimerA0.1 compare value for duty cycle
    else if (duty == 75)                                   // 75% duty cycle
        TA0CCR1   = 95;                                    // TimerA0.1 compare value for duty cycle

    TA0CCTL1  |= OUTMOD_7;                                 // Output mode to PWM reset/set
    TA0CTL |= MC_1 | TACLR;                                // Timer up mode to start PWM
    P1SEL |= TRIG2;                                        // Output PWM on gpio
}// end Trigger2 PWM 125kHz 25% or 75% duty cycle

// Set trigger for reading drv3901q1 Registers
void drv3901q1_RegistersRead()
{
    RegReadPeriod++;
    if (RegReadPeriod == REG_READ_PERIOD)
    {
        RegReadPeriod = 0;

        if (gBrushedObj.AutoRead)                          // Periodic read Registers and update Register memory buffer if Auto read in GUI is not off
        {
            ReadCMD = 1;
        }
    }
}// End Set trigger for reading drv3901q1 Registers
