/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
/*******************************************************************************
 *
 *  uart.c - c file for UART communication via UART to USB to the PC GUI app
 *
 ******************************************************************************/
#include "drv3901-q1/drv3901-q1.h"
#include "uart.h"

unsigned int txData;
extern void receivedDataCommand(unsigned char receiveData);

// This function initializes configuration for UCA0 USCI in UART communication mode with 9600 BAUD
void uartInit(void)
{
    P3SEL |= RX | TX;                            // P3.5 = UART_RXD, P3.4 = UART_TXD

    UCA0CTL1 |= UCSSEL_2;                        // SMCLK UART clock input

    // 16MHz 9600 baud
    UCA0BR0 = 104;                               // Baud rate divider low byte
    UCA0BR1 = 0;                                 // Baud rate divider high byte

    // Modulation UCBRSx = 0, oversampling mode enabled
    UCA0MCTL = 49;                               // Baud rate divider modulation value

    // Initialize USCI state machine
    UCA0CTL1 &= ~UCSWRST;

    // Enable USCI_A0 RX interrupt
    IE2 |= UCA0RXIE;                             // UART RX interrupt enable
}// end uartInit()

void uartTxByte(unsigned char byte)
{
    // USCI_A0 TX buffer ready?
    while (!(IFG2 & UCA0TXIFG));

    // transmit data
    UCA0TXBUF = byte;
}// End uartTxByte()

// USCI Receive interrupt service routine to receive the MDBU data packets
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
    receivedDataCommand(UCA0RXBUF);              // Parse received command immediately
    P3OUT ^= GUI_STATUS;                         // Toggle GUI_STATUS LED for every UART RX interrupt
}// end USCI0RX_ISR()
